#include "kmmimeparttree.h"
#include "kmreaderwin.h"
#include "partNode.h"
#include <qheader.h>
#include <kdebug.h>
#include <klocale.h>
#include <qpopupmenu.h>
#include <kfiledialog.h>
#include <qfile.h>
#include <kmessagebox.h>


KMMimePartTree::KMMimePartTree( KMReaderWin* readerWin,
                                QWidget* parent,
                                const char* name )
    : QListView(  parent, name ),
      mReaderWin( readerWin )
{
    addColumn( i18n("description") );
    addColumn( i18n("type") );
    addColumn( i18n("encoding") );
    addColumn( i18n("size") );
    setColumnAlignment(3, Qt::AlignRight);
    setResizeMode( QListView::LastColumn );
    connect( this, SIGNAL( clicked( QListViewItem* ) ), 
             this, SLOT( itemClicked( QListViewItem* ) ) );
    connect( this, SIGNAL( contextMenuRequested( QListViewItem*, 
                                                 const QPoint&, int ) ),
             this, SLOT( itemRightClicked( QListViewItem*, const QPoint& ) ) );
    setRootIsDecorated( false );
    setShowToolTips( true );
    setAllColumnsShowFocus( true );
    setShowToolTips( true );
    setSorting(-1);
}

void KMMimePartTree::itemClicked( QListViewItem* item )
{
    KMMimePartTreeItem* i = dynamic_cast<KMMimePartTreeItem*>( item );
    if ( 0L == i ) {
        kdDebug(5006) << "Item was not a KMMimePartTreeItem!" << endl;
    }
    else {
        kdDebug(5006) << "\n**\n** KMMimePartTree::itemClicked() **\n**" << endl;
        mReaderWin->parseObjectTree( i->node(), true );
    }
}


void KMMimePartTree::itemRightClicked( QListViewItem* item, 
                                       const QPoint& point )
{
    mCurrentContextMenuItem = dynamic_cast<KMMimePartTreeItem*>( item );
    QPopupMenu* popup = new QPopupMenu;
    popup->insertItem( i18n( "Save &As..." ), this, SLOT( slotSaveAs() ) );
    popup->insertItem( i18n( "Save As &Encoded..." ), this,
                       SLOT( slotSaveAsEncoded() ) );
    popup->exec( point );
    mCurrentContextMenuItem = 0;
}


void KMMimePartTree::slotSaveAs()
{
    if( mCurrentContextMenuItem ) {
        // this should always be true
        QString filename = KFileDialog::getSaveFileName( QString::null,
                                                         QString::null,
                                                         this, QString::null );
        if( !filename.isEmpty() ) {
            if( QFile::exists( filename ) ) {
                if( KMessageBox::warningYesNo( this,
                                               i18n( "A file with this name already exists. Do you want to overwrite it?" ),
                                               i18n( "KMail Warning" ) ) ==
                    KMessageBox::No )
                    return;
            }
            QFile file( filename );
            if( file.open( IO_WriteOnly ) ) {
                QDataStream ds( &file );
                QByteArray cstr( mCurrentContextMenuItem->node()->msgPart().bodyDecodedBinary() );
                ds.writeRawBytes( cstr, cstr.size() );
                file.close();
            } else
                KMessageBox::error( this, i18n( "Could not write the file" ),
                                    i18n( "KMail Error" ) );
        }
    }
}


void KMMimePartTree::slotSaveAsEncoded()
{
    if( mCurrentContextMenuItem ) {
        // this should always be true
        QString filename = KFileDialog::getSaveFileName( QString::null,
                                                         QString::null,
                                                         this, QString::null );
        if( !filename.isEmpty() ) {
            if( QFile::exists( filename ) ) {
                if( KMessageBox::warningYesNo( this,
                                               i18n( "A file with this name already exists. Do you want to overwrite it?" ),
                                               i18n( "KMail Warning" ) ) ==
                    KMessageBox::No )
                    return;
            }
            QFile file( filename );
            if( file.open( IO_WriteOnly ) ) {
                QDataStream ds( &file );
                QCString cstr( mCurrentContextMenuItem->node()->msgPart().body() );
                ds.writeRawBytes( cstr, cstr.length() );
                file.close();
            } else
                KMessageBox::error( this, i18n( "Could not write the file" ),
                                    i18n( "KMail Error" ) );
        }
    }
}


KMMimePartTreeItem::KMMimePartTreeItem( KMMimePartTree& parent,
                                        partNode* node,
                                        const QString& labelDescr,
                                        QString labelCntType,
                                        QString labelEncoding,
                                        QString labelSize )
    : QListViewItem( &parent, labelDescr,
                     labelCntType,
                     labelEncoding,
                     labelSize ),
      mPartNode( node )
{
}

KMMimePartTreeItem::KMMimePartTreeItem( KMMimePartTreeItem& parent,
                                        partNode* node,
                                        const QString& labelDescr,
                                        QString labelCntType,
                                        QString labelEncoding,
                                        QString labelSize )
    : QListViewItem( &parent, labelDescr,
                     labelCntType,
                     labelEncoding,
                     labelSize ),
      mPartNode( node )
{
}


